// IBIS Command Sender
// (c) Pavel Kaspar
// kaspar@pk.cz
// version 0.91; 2.1.2022

#include <SoftwareSerial.h>

#define Rx 0
#define Tx 1
SoftwareSerial mySerial(Rx, Tx);

/* ATTiny85 connections
 Leg  Function
 1    PB5, Reset, external pull-up
 2    PB3, no connection
 3    PB4, no connection 
 4    GND
 5    PB0 - Rx
 6    PB1 - Tx
 7    PB2 - Tx Loop => Input
 8    +5V
*/

byte Funkce;                     // Funkce - vyplyva z nastaveni konfiguracnich jumperu (u prevodniku verze 2) 
byte Parita;                     // Pomocna promenna pro vypocet paritniho byte

byte PrevodniTabulka[128] = {    // Pomocna tabulka pro vytvoreni sude parity prenaseneho znaku (MSB)
  0x00,0x81,0x82,0x03,0x84,0x05,0x06,0x87,0x88,0x09,0x0A,0x8B,0x0C,0x8D,0x8E,0x0F,
  0x90,0x11,0x12,0x93,0x14,0x95,0x96,0x17,0x18,0x99,0x9A,0x1B,0x9C,0x1D,0x1E,0x9F,
  0xA0,0x21,0x22,0xA3,0x24,0xA5,0xA6,0x27,0x28,0xA9,0xAA,0x2B,0xAC,0x2D,0x2E,0xAF,
  0x30,0xB1,0xB2,0x33,0xB4,0x35,0x36,0xB7,0xB8,0x39,0xBA,0xBB,0x3C,0xBD,0xBE,0x3F,
  0xC0,0x41,0x42,0xC3,0x44,0xC5,0xC6,0x47,0x48,0xC9,0xCA,0x4B,0xCC,0x4D,0x4E,0xCF,
  0x50,0xD1,0xD2,0x53,0xD4,0x55,0x56,0xD7,0xD8,0x59,0x5A,0xDB,0x5C,0xDD,0xDE,0x5F,
  0x60,0xE1,0xE2,0x63,0xE4,0x65,0x66,0xE7,0xE8,0x69,0x6A,0xEB,0x6C,0xED,0xEE,0x6F,
  0xF0,0x71,0x72,0xF3,0x74,0xF5,0xF6,0x77,0x78,0xF9,0xFA,0x7B,0xFC,0x7D,0x7E,0xFF,
  };

// Prikazy
void Posli(byte Zn){
  Zn = Zn & 0x7F;
  Zn = PrevodniTabulka[Zn];
  Parita = Parita ^ Zn;
  mySerial.write(Zn); 
  delay(1);                       // Nahrada druheho STOP bitu serioveho prenosu
}

void Posli_CR_a_Paritu(){         // Posli CR a zaverecny paritni byte
  Posli(0x8D);
  Parita = ~Parita;
  mySerial.write(Parita); 
  delay(1);  
}

void PosliString(String Str){     // Pouziti Stringu neni doporucovano u malych mikrokontroleru
byte i;                           // V dulezitych aplikacich pouzit jiny datovy typ
  Parita = 0;
  for (int i=0; i<Str.length(); i++) {
    Posli(Str[i]); }
  Posli_CR_a_Paritu(); 
}

byte Nacti_Jumpery(){             // Nacte konfiguracni jumpery (u prevodniku verze 2)
byte Jumper;
byte Fce;
  Fce = digitalRead(0);
  Jumper = digitalRead(2);
  Fce = Fce + 2 * Jumper;  
  Jumper = digitalRead(3);
  Fce = Fce + 4 * Jumper;
  Jumper = digitalRead(4);
  Fce = Fce + 8 * Jumper;
  Fce = ~Fce;                     // Negace -> Strep = "1"
  Fce = Fce & 0x0F;
  return Fce;  
}

// Prime prikazy
void l000() {                     // Prazdny
  Parita = 0;
  Posli(0x6C);
  Posli(0x30);
  Posli(0x30);
  Posli(0x30);
  Posli_CR_a_Paritu();    
}

void l999() {                     // Linka 999 - napis BUSE
  Parita = 0;
  Posli(0x6C);
  Posli(0x39);
  Posli(0x39);
  Posli(0x39);
  Posli_CR_a_Paritu();     
}

// Inicializace
void setup() {
  pinMode(Rx, INPUT_PULLUP);
  pinMode(Tx, OUTPUT);
  pinMode(2, INPUT_PULLUP);
  pinMode(3, INPUT_PULLUP);
  pinMode(4, INPUT_PULLUP); 
//  pinMode(5, INPUT_PULLUP); 
  mySerial.begin(1200); 
  Parita = 0;
  Funkce = Nacti_Jumpery();
}

// Main
void loop() {
  delay(2000);      // Cekani na start pripojenych zarizeni
    
  Funkce = 255;
  switch (Funkce) {
    case 0: 
      PosliString("l979");  // Linka 979 - DP Podebrady  
      break;
    case 1: 
      PosliString("l971");  // Linka 971 - Vesele Vanoce  
      break;
    case 2: 
      PosliString("l974");  // Linka 974 - Vesele Velikonoce 
      break;
    case 3: 
      PosliString("l973");  // Linka 973 - Vsechno nejlepsi do noveho roku 
      break;
    case 4: 
      PosliString("l953");  // Linka 953 - Zajezd Podebrady
      break;
    case 5: 
      PosliString("l960");  // Linka 960 - Stasny novy rok 
      break;
    case 6: 
      PosliString("l972");  // Linka 972 - Stastne a vesele Vanoce 
      break;
    case 7: 
      PosliString("l978");  // Linka 978 - Happy Birthday 
      break;  
    case 8:
      PosliString("l951");  // Linka 951 - Zajezd (deti) 
      break;       
    case 9: 
      PosliString("l955");  // Linka 955 - Zvlastni jizda 
      break;      
    case 10: 
      PosliString("l958");  // Linka 958 - Mimoradna linka
      break; 
    case 11:
      PosliString("l963");  // Linka 963 - PF 2023
      break; 
    case 12: 
      PosliString("l964");  // Linka 964 - PF 2024
      break;      
    case 13: 
      PosliString("l965");  // Linka 965 - PF 2025 
      break;
    case 14: 
      PosliString("l966");  // Linka 966 - PF 2026 
      break;      
    case 15:   
      PosliString("l987");  // Linka 987 - Tramvaj do Podebrad
      break;
    default:  
      l000();  
      delay(3000);
      l999();
      delay(3000);
      for (int i=0; i<1000; i++) {
        if (i >= 100) {
          PosliString("l"+String(i));
        }
        else {
          if (i >= 10) {
          PosliString("l0"+String(i));
          }
          else {
          PosliString("l00"+String(i));
          }
        }
        delay(3000);  
      }   // loop
  }  // switch
  delay(30000);
}


